/*

DESCRIPTION:
    Main include file for digital PWM demonstrator 48V inverter.

SYNTAX:  n/a

KNOWN BUGS:  n/a.

COPYRIGHT:  (c) 2015, Brian Cornell, all rights reserved.

DEPENDENCIES:  none

VERSION:  1.0, development release

MODIFICATION HISTORY:

    25 Mar 18, BWC, initial development release.


*/



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Version constants & definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define VERMAJOR    1
#define VERMINOR    0
#define VERBUILD    51


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Necessary includes
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifndef UINT_MAX
#include <limits.h>
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Conditional compilation.  Uncomment directive to enable / disable features.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Uncomment to remove limit checks & not save errors to NVM in run-time loop
//#define _NO_LIMITS


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * System wide constants & definitions.  Types:
 *  + DC=Duty Cycle
 *  + EN=ENable bit
 *  + RT=RaTio of a voltage presented to PIC, internal or external
 *  + LM=LiMiting operational value
 *  + TM=TiMing
 *  + MP=Peripheral port bit MaPping
 *  + CH=character
 *  + IN=INitializing value
 *  + VL=VoLtage
 *  + QY=quantity (count) of something
 *  + CD=command data
 *  + BL=boolean value
 *  + MK=bit MasK
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define S_DC_MIN        0x08                //Minimum allowable duty cycle
#define S_DC_MAX        0X98                //Maximum allowable duty cycle
#define S_DC_RUN        0x98                //Running duty cycle
#define S_DC_MIN_P      ((S_DC_MIN * 100) / (CFG_PWM_5_FRQ + 1))
#define S_DC_MAX_P      ((S_DC_MAX * 100) / (CFG_PWM_5_FRQ + 1))
#define S_DC_RUN_P      ((S_DC_RUN * 100) / (CFG_PWM_5_FRQ + 1))
#define S_EN_ADC        0x01                //Enable bit position in ADCON0 register
#define S_RT_VDD        2                   //Vdd ratio rendered by DAC
#define S_RT_INV        11                  //Inverter DC+ bus voltage ratio
#define S_RT_VCC        5                   //Inverter Vcc voltage ratio
#define S_RT_ICS        1                   //Inverter current sense S&H voltage ratio
#define S_RT_TMP        1                   //Inverter NTC (temp) sense voltage ratio
#define S_RT_SEN        1                   //Ratio for Vcc sense voltage for contro loop
#define S_RT_REF        1                   //Ratio for reference voltage for control loop
#define S_VL_INV_MIN    10000               //Minimum DC+ bus voltage
#define S_VL_INV_MAX    16000               //Maximum DC+ bus voltage
#define S_VL_INV_RSTART 12000               //Restart (from UVLO) DC+ bus voltage
#define S_LM_VCC_MIN    9000                //Minimum VCC voltage to allow NVM writes
#define S_TM_RST        200                 //# mS to wait before reseting PIC
#define S_TM_INT        300                 //# mS for system to stabilize at power-up
#define S_TM_RELAY      25                  //# mS for inverter DC+ inrush after relay on
#define S_TM_START      1000                //# mS to wait to start sine after inverter
#define S_MP_CSENS      C2OUT               //CM2 output bit for over-current sense
#define S_LM_CLLOW      1                   //Lowest allowable current limit value
#define S_LM_CLHIG      1023                //Highest allowable current limit value
#define S_LM_VRLOW      1                   //Lowest allowable Vref
#define S_LM_VRHIG      1023                //Highest allowable Vref
#define S_BL_AUTOD      0                   //Autostart disabled
#define S_BL_AUTOI      1                   //Autostart inverter only
#define S_BL_AUTOS      2                   //Autostart inverter & sine
#define S_LM_OVC_HYS    160                 //Hysteresis for over-current event limit
#define S_LM_OVP_HYS    160                 //Hysteresis for over-power event limit
#define S_LM_UVL_HYS    160                 //Hysteresis for DC bus under volt event limit
#define S_LM_PCON_EX    0xFA                //PCON bits to examine at boot
#define S_LM_PCON_BT    0x18                //PCON expected value at boot
#define S_LM_STAT_EX    0x18                //STATUS bits to examine at boot
#define S_LM_STAT_BT    0x18                //STATUS expected value at boot
#define S_TM_RLY_ON     100                 //# uS to wait for relay ON signal
#define S_LM_RLY_ON     10                  //# S_TM_RLY_ON periods before relay ON timeout
#define S_TM_RLY_OFF    100                 //# uS to wait for relay OFF signal
#define S_LM_RLY_OFF    50                  //# S_TM_RLY_ON periods before relay OFF timeout
#define S_TM_SIN_ON     100                 //# uS to wait for sine ON signal
#define S_LM_SIN_ON     20                  //# S_TM_SIN_ON periods before sine ON timeout
#define S_TM_SIN_OFF    100                 //# uS to wait for sine OFF signal
#define S_LM_SIN_OFF    100                 //# S_TM_SIN_OFF periods before sine OFF timeout
#define S_CH_OPEN       'O'                 //Display indicator for open loop
#define S_CH_CLOSED     'C'                 //Display indicator for closedloop
#define S_TM_DCDELAY    500                 //# uS to wait between each DC increment
#define S_LM_PRGLOW     0                   //Lowest PRG ramp rate value
#define S_LM_PRGHIG     31                  //Hightest PRG ramp rate value
#define S_CH_ANALOG     'A'                 //Display indicator for analog control loop
#define S_CH_DIGITAL    'D'                 //Display indicator for digitalcontrol loop
#define S_TM_MONINTVL   100                 //# SysTimer clock ticks for monitor updates (1S)
#define S_IN_SAMPLES        1               //Starting value for measured samples counter
#define S_LM_SAMPLES        120             //Number of measured samples to average
#define S_TM_INVON_COG      5               //mS delay before measuring COG time on inverter start
#define S_VL_MV_PER_AMP     15              //Current sense mV/A represented as integer
#define S_LM_PWR_MAX        500             //Maximum allowable power in watts
#define S_LM_PWR_100        350             //Continuous full power rating in watts
#define S_LM_PWR_BOOST      ((S_LM_PWR_100 * 11) / 10)      //<110% of rated continuous power (see hack note below)
#define S_LM_PWR_75         ((S_LM_PWR_100 * 75) / 100)     //<75% of rated continuous power
#define S_LM_PWR_50         ((S_LM_PWR_100 * 50) / 100)     //<50% of rated continuous power
#define S_LM_PWR_25         ((S_LM_PWR_100 * 25) / 100)     //<25% of rated continuous power
#define S_MK_FAULT          0x01            //Fault indicator on
#define S_MK_RUN            0x02            //Run indicator on
#define S_MK_BOOST          0x04            //Boost indicator on
#define S_MK_L100           0x08            //100% indicator on
#define S_MK_L75            0x10            //75% indicator on
#define S_MK_L50            0x20            //50% indicator on
#define S_MK_L25            0x40            //25% indicator on
#define S_VL_TMP_OVH        1500            //Inverter NTC voltage representing overheat temp

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                         Hack Notes for Constants
 * 
 * S_LM_PWR_BOOST:  trailing zero on multiplier / divide is dropped to work around
 * compiler treating as signed 16-bit integer.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                         Object sizes & positions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SIZ_INT_ADC 2                   //# bytes in ADC integer
#define POS_HIG_ADC 1                   //ADC result high byte
#define POS_LOW_ADC 0                   //ADC result low byte


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Analog channel mapping:  channel=port=pin.  The values assigned to these
 * constants should match those used by ADC.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define ANLCAN0     0x00                //AN0=RA0=2
#define ANLCAN1     0x04                //AN1=RA1=3
#define ANLCAN2     0x08                //AN2=RA2=4
#define ANLCAN3     0x0C                //AN3=RA3=5
#define ANLCAN4     0x10                //AN4=RA5=7
#define ANLCAN8     0x20                //AN8=RB2=23
#define ANLCAN9     0x24                //AN9=RB3=24
#define ANLCAN10    0x28                //AN10=RB1=22
#define ANLCAN11    0x2C                //AN11=RB4=25
#define ANLCAN12    0x30                //AN12=RB0=21
#define ANLCAN13    0x34                //AN13=RB5=26
#define ANLCAN14    0x38                //AN14=RC2=13
#define ANLCAN15    0x3C                //AN15=RC3=14
#define ANLCAN16    0x40                //AN16=RC4=15
#define ANLCAN17    0x44                //AN17=RC5=16
#define ANLCAN18    0x48                //AN18=RC6=17
#define ANLCAN19    0x4C                //AN19=RC7=18
#define ANLCANS1    0x84                //Switched AN1=RA1=3
#define ANLCANS10   0xA8                //Switched AN10=RB1=22
#define ANLCANS18   0xC8                //Switched AN18=RC6=17


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * PPS input pin definitions.  Map for MPU and use to assign peripheral input
 * pins.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PPSRA0      0x00                //RA0
#define PPSRA1      0x01                //RA1
#define PPSRA2      0x02                //RA2
#define PPSRA3      0x03                //RA3
#define PPSRA4      0x04                //RA4
#define PPSRA5      0x05                //RA5
#define PPSRA6      0x06                //RA6
#define PPSRA7      0x07                //RA7
#define PPSRB0      0x08                //RB0
#define PPSRB1      0x09                //RB1
#define PPSRB2      0x0A                //RB2
#define PPSRB3      0x0B                //RB3
#define PPSRB4      0x0C                //RB4
#define PPSRB5      0x0D                //RB5
#define PPSRB6      0x0E                //RB6
#define PPSRB7      0x0F                //RB7
#define PPSRC0      0x10                //RC0
#define PPSRC1      0x11                //RC1
#define PPSRC2      0x12                //RC2
#define PPSRC3      0x13                //RC3
#define PPSRC4      0x14                //RC4
#define PPSRC5      0x15                //RC5
#define PPSRC6      0x16                //RC6
#define PPSRC7      0x17                //RC7


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                           PPS & direct pin mapping
 * 
 * For PPS output pin assignments both aliases for pin registers and constants 
 * mapping what peripheral connects to the pin.
 *      Pin register alias convention:  PPS_P_XXX_YY where
 *          XXX=peripheral name
 *          YY=channel or other detail; can be omitted
 *      Peripheral mapping convention:  PPS_M_XXX_YY same as alias
 * 
 * Input pins only require mapping constant.
 * 
 * For pins accessed directly or assigned in a peripheral's register (like ADC)
 * use the following convention:  PIN_X_YYYYYY where
 *          X=Input or Output
 *          YYYYYY=user assigned mnemonic describing function
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PPS_P_SIDE_A_H  RC4PPS          //COG1 A maps to bridge side A high, pin 15
#define PPS_P_SIDE_A_L  RC5PPS          //COG1 D maps to bridge side A low, pin 16
#define PPS_P_SIDE_B_H  RB4PPS          //COG1 B maps to bridge side B high, pin 25
#define PPS_P_SIDE_B_L  RB5PPS          //COG1 C maps to bridge side B low, pin 26
#define PPS_M_COG1_A    0x05            //Output COG1 A signal on selected pin
#define PPS_M_COG1_B    0x06            //Output COG1 B signal on selected pin
#define PPS_M_COG1_C    0x07            //Output COG1 C signal on selected pin
#define PPS_M_COG1_D    0x08            //Output COG1 D signal on selected pin

#define PPS_M_LAT_RC5   0x00            //Output port latch value on side A low
#define PPS_M_LAT_RB5   0x00            //Output port latch value on side B low

#define PPS_M_TMR_5G    PPSRC4          //Timer5 gate pin=RC4
#define PIN_I_IVDCPV    ANLCAN3         //ADC sense - inverter DC+
#define PIN_I_IVVCCV    ANLCAN12        //ADC sense - inverter VCC
#define PIN_I_IVCSSH    ANLCAN10        //ADC sense - inverter current sense S&H
#define PIN_I_INVTMP    ANLCAN8         //ADC sense - inverter temp

#define PIN_O_SIDE_A_L  RC5             //Direct control of side A low switch
#define PIN_O_SIDE_B_L  RB5             //Direct control of side B low switch
#define PIN_O_SINE      RA2             //Enable for digital sine generator
#define PIN_I_SINE      RA4             //Acknowledge for digital sine generator
#define PIN_O_RELAY     RA0             //Enable for SSR
#define PIN_I_RELAY     RA1             //Acknowledge for SSR

//LED output pins for fault, run, and power levels
#define PIN_O_FAULT     RA7             //Fault
#define PIN_O_RUN       RA5             //Run
#define PIN_O_BOOST     RC3             //Boost power
#define PIN_O_L100      RC2             //100% power
#define PIN_O_L75       RC1             //75% power
#define PIN_O_L50       RC0             //50% power
#define PIN_O_L25       RA6             //25% power


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Port/pin configuration:  I/O, analog/digital, slew rate, etc.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Port I/O:  1=input, 0=output
#define IO_PORTA    0b00011010
#define IO_PORTB    0b10001101
#define IO_PORTC    0b10000000
//Port Analog/Digital:  1=analog, 0=digital
#define AD_PORTA    0b00001000
#define AD_PORTB    0b00001111
#define AD_PORTC    0b11000000
//Port weak pull-up:  1=enabled, 0=disabled
#define WP_PORTA    0b00000000
#define WP_PORTB    0b00000000
#define WP_PORTC    0b00000000
//Open drain:  1=open drain, 0=push-pull
#define OD_PORTA    0b00000000
#define OD_PORTB    0b00000000
#define OD_PORTC    0b00000000
//Slew rate:  1=limited, 0=maximum
#define SR_PORTA    0b00000000
#define SR_PORTB    0b00000000
#define SR_PORTC    0b00000000
//Input level:  1=ST, 0=TLL
#define IL_PORTA    0b00000000
#define IL_PORTB    0b00000000
#define IL_PORTC    0b00000000


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                            Macro functions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define clearCmdResult()    cmdGood = TRUE
#define setCmdOK()          cmdGood = TRUE
#define setCmdErr()         cmdGood = FALSE
#define printCmdResult()    if (prnResult) if (cmdGood) printf(msgOk); else printf(msgErr)
#define printCmdSys()       if (prnResult) printf("%s%s\n", lblSys, StsList[Config.sts].desc)


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *              ADC configuration & acquisition constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//AD acquisition configuration & control constants
#define ADCCFGC1    0xD3                //Result right justified, Fosc/16, Vss, FVR
#define ADCCFGC2    0x00                //No auto conversion
#define ADCACQTIM   10                  //# uS required for sample & hold cap to acquire
#define ADCMAXVAL   1023                //Maximum value 10-bit ADC can return

//Internal channels (peripherals)
#define ADC_CH_FVR      0xFC            //FVR
#define ADC_CH_DAC_1    0xF8            //DAC1
#define ADC_CH_TMP      0xF4            //Temperature indicator
#define ADC_CH_DAC_2    0xF0            //DAC2
#define ADC_CH_DAC_3    0xEC            //DAC3
#define ADC_CH_DAC_4    0xE8            //DAC4
#define ADC_CH_DAC_5    0xE4            //DAC5
#define ADC_CH_DAC_7    0xDC            //DAC7


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Flash program memory definitions.  For non-volatile storage.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define FLASHROWSZ      32              //Size of flash row: PIC specific
//Flash memory unlock sequence
#define unlockSeq() PMCON2 = 0x55; PMCON2 = 0xAA; WR = TRUE; NOP(); NOP()
#define FLASH_AR_AUTO   0               //Autostart setting
#define FLASH_AR_STS    1               //Last status code
#define FLASH_AR_OLDC   2               //PWM open loop DC register value
#define FLASH_AR_OLDC_P 3               //Open loop DC %
#define FLASH_AR_OPENL  4               //Non-zero if Open loop
#define FLASH_AR_VREF_H 5               //High byte of Vref DAC word
#define FLASH_AR_VREF_L 6               //Low byte of Vref DAC word
#define FLASH_AR_LOOP_D 7               //Digital control loop indicator


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Control constants for function ReadVoltage
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define FRV_ADC_AQT 20                  //# uS required for sample & hold cap to acquire
#define FRV_ADC_V1V 250                 //ADC value for 1V when Vref=4.096


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Timer2 configuration:  provides interrupt timing for background caretaker that
 * performs system monitor & control.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_TMR_2_CLK   0x05            //MFINTOSC
#define CFG_TMR_2_CON   0x19            //Off, 1:2 prescale, 1:10 post scale
#define CFG_TMR_2_HLT   0xA0            //Prescale syncs Fosc/4, on bit syncs Fosc/4, free running
#define CFG_TMR_2_RST   0x00            //No external reset sources
#define CFG_TMR_2_PR    250             //Period value to obtain 10mS period

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Timer5 configuration and control constants for function getCOGtime.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_TMR_5_C     0x41            //Fosc, 1:1 prescale, sync, on
#define CFG_TMR_5_G     0xD0            //Gate enable, 1 polarity, single pulse, gate pin

#define FGCT_MAXCNT     160             //Maximum count for 10uS
#define FGCT_1USCNT     16              //Count for 1uS
#define FGCT_WAITTM     30              //# uS to wait before checking go/done bit


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * FVR, DAC, and function Initialize configuration constants
 * DAC1:  bridge over-current sense
 * DAC4:  Vdd sense
 * DAC5:  error amp
 * DAC7:  ramp generator (PRG)
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_FVR_C       0xCF            //Enabled, no temp sense, CDAFVRx4, ADFVRx4

//DAC1:  2.1V (525 decimal, approximately 140A)
#define CFG_DAC_1_C     0x88            //Enabled, right justified, FVR buffer 2, Vss
#define CFG_DAC_1_VH    0x02            //Output high byte
#define CFG_DAC_1_VL    0x0D            //Output low byte

//DAC4:  Vdd/2
#define CFG_DAC_4_C     0x80            //Enabled, Vdd, Vss
#define CFG_DAC_4_V     0x10            //Output byte

//DAC5:  2.6V=inverter Vcc ~ 13V
#define CFG_DAC_5_C     0x88            //Enabled, right justified, FVR buffer 2, Vss
#define CFG_DAC_5_VH    0x02            //Output high byte
#define CFG_DAC_5_VL    0x8A            //Output low byte

//DAC7:  0V
#define CFG_DAC_7_C     0x80            //Enabled, Vdd, Vss
#define CFG_DAC_7_V     0x00            //Output byte

#define FI_FVR_DLY      30              //Time delay in uS for FVR buffer to stabilize


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Current sense sample & hold OPA2 configuration
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_OPA_2_C     0x11            //Disabled, unity gain, override tri-states output
#define CFG_OPA_2_O     0x15            //CLC2 is override source
#define CFG_OPA_2_N     0x00            //Not used with unity gain
#define CFG_OPA_2_P     0x00            //Connect (+) to OPA2IN0+


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Error amplifier OPA3 configuration
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_OPA_3_C     0x80            //Enabled, override disabled
#define CFG_OPA_3_N     0x00            //Connect (-) to OPA3IN0- pin
#define CFG_OPA_3_P     0x02            //Connect (+) to DAC5_out


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure comparator 2 for inverter bridge over-current sense.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_2_C0     0xAA            //Enabled, normal pwr, ZLF on, HYS on
#define CFG_CM_2_C1     0x01            //Interrupt on falling edge
#define CFG_CM_2_N      0x02            //Connect (-) to C2IN2- pin (24)
#define CFG_CM_2_P      0x05            //Connect (+) to DAC1 output


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure comparator 5 as falling event input to CLC1; (-) from error amp, (+) 
 * from PRG.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_5_C0     0xBE            //Enabled, output inverted, normal pwr, ZLF on, HYS on
#define CFG_CM_5_C1     0x00            //No interrupts
#define CFG_CM_5_N      0x01            //Connect (-) to C5IN1- pin (17)
#define CFG_CM_5_P      0x03            //Connect (+) to PRG3 output


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          PWM configurations
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Configure 16-bit PWM5:  master and sets max DC
#define PWM5CFG 0b10000000      //Enabled, active high output, standard mode
#define PWM5CIE 0b00000000      //No interrupts
#define PWM5CCL 0b00000001      //No prescale, use HFINTOSC as clock source
#define PWM5CLD 0b10000000      //Load triggering is immediate
#define PWM5COT 0b00000000      //Offset trigger source is independent run mode
#define PWM5CPH 0x0000          //Phase count
#define CFG_PWM_5_FRQ   0x009F  //Period (frequency)
#define PWM5COC 0x0000          //Offset count
#define PWM5CTM 0x0000          //Starting value for timer

//Configure 16-bit PWM6:  slave to PWM5 and sets min DC
#define PWM6CFG 0b10000000      //Enabled, active high output, standard mode
#define PWM6CIE 0b00000000      //No interrupts
#define PWM6CCL 0b00000001      //No prescale, use HFINTOSC as clock source
#define PWM6CLD 0b10000000      //Load triggering is immediate
#define PWM6COT 0b00100000      //Slave run mode with offset triggered sync start from PWM5
#define PWM6CPH 0x0000          //Phase count
#define PWM6CDC 0x0008          //Duty cycle (5%)
#define PWM6CPD 0x009F          //Period (frequency)
#define PWM6COC 0x0000          //Offset count
#define PWM6CTM 0x0000          //Starting value for timer

//Configure PWM11:  slave to PWM5 to time current sense falling event blanking
#define CFG_PWM_11_CON      0x80                //Enabled, active high output, standard mode
#define CFG_PWM_11_INTE     0x00                //No interrupts
#define CFG_PWM_11_CLK      0x01                //No prescale, use HFINTOSC as clock source
#define CFG_PWM_11_LDCON    0x80                //Load buffers at end of current period
#define CFG_PWM_11_OFCON    0x20                //Slave run mode with offset triggered sync start from PWM5
#define CFG_PWM_11_PHASE    0x00                //Phase
#define CFG_PWM_11_DUTYC    0x000D              //Duty cycle (blanking time - 812nS)
#define CFG_PWM_11_PERIOD   0x009F              //Period
#define CFG_PWM_11_OFFSET   0x0000              //Offset count
#define CFG_PWM_11_TIEMR    0x0000              //Starting value for timer


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC1 as OR gate to control COG1 falling event triggers
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_1_CON       0x00                //Disabled, no interrupts, AND-OR
#define CFG_CLC_1_POL       0x00                //No inverted polarities
#define CFG_CLC_1_SEL_0     0x0C                //Comparator 5 out selected for d1T/g1
#define CFG_CLC_1_SEL_1     0x0C                //Comparator 5 out selected for d2T/g12
#define CFG_CLC_1_SEL_2     0x21                //PWM6 out selected for d3T/g3
#define CFG_CLC_1_SEL_3     0x21                //PWM6 out selected for d4T/g4
#define CFG_CLC_1_GLS_0     0x02                //d1T is gated into g1
#define CFG_CLC_1_GLS_1     0x08                //d2T is gated into g2
#define CFG_CLC_1_GLS_2     0x20                //d3T is gated into g3
#define CFG_CLC_1_GLS_3     0x80                //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC2 as OR gate to control OPA2 tri-state for current sense S&H
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_2_CON       0x00                //Disabled, no interrupts, AND-OR
#define CFG_CLC_2_POL       0x00                //No inverted polarities
#define CFG_CLC_2_SEL_0     0x10                //COG1A selected for d1T/g1
#define CFG_CLC_2_SEL_1     0x10                //COG1A selected for d2T/g2
#define CFG_CLC_2_SEL_2     0x11                //COG1B selected for d3T/g3
#define CFG_CLC_2_SEL_3     0x11                //COG1B selected for d4T/g4
#define CFG_CLC_2_GLS_0     0x02                //d1T is gated into g1
#define CFG_CLC_2_GLS_1     0x08                //d2T is gated into g2
#define CFG_CLC_2_GLS_2     0x20                //d3T is gated into g3
#define CFG_CLC_2_GLS_3     0x80                //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC3 as OR gate to control current sense blanking
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_3_CON       0x00                //Disabled, no interrupts, AND-OR
#define CFG_CLC_3_POL       0x00                //No inverted polarities
#define CFG_CLC_3_SEL_0     0x22                //PWM11 selected for d1T/g1
#define CFG_CLC_3_SEL_1     0x22                //PWM11 selected for d2T/g2
#define CFG_CLC_3_SEL_2     0x09                //COG1B selected for d3T/g3
#define CFG_CLC_3_SEL_3     0x09                //COG1B selected for d4T/g4
#define CFG_CLC_3_GLS_0     0x02                //d1T is gated into g1
#define CFG_CLC_3_GLS_1     0x08                //d2T is gated into g2
#define CFG_CLC_3_GLS_2     0x20                //d3T is gated into g3
#define CFG_CLC_3_GLS_3     0x80                //d4T is gated into g4


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure Complementary Output Generator 1 (COG1).  Start with peripheral
 * config.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_COG_1_CON0          0xD5            //Enable, reg load next event, HFINTOSC clock, push/pull
#define CFG_COG_1_CON1          0x00            //Clock used for dead band, all outputs active high

//Rising events
#define CFG_COG_1_REI0_NONE     0x00            //No rising event inputs for RIS0
#define CFG_COG_1_REI1_PWM5     0x02            //PWM5 rising event input for RIS1
#define CFG_COG_1_REM0_NONE     0x00            //No rising event for RSIM0
#define CFG_COG_1_REM1_PWM5     0x02            //PWM5 low->high for RSIM1

//Falling events
#define CFG_COG_1_FEI0_NONE     0x00            //No falling events
#define CFG_COG_1_FEI1_OLOOP    0x42            //Open loop:  CLC3 & PWM5 FE input
#define CFG_COG_1_FEI1_CLOOP    0x52            //Closed loop:  CLC1, CLC3, PWM5 FE inputs
#define CFG_COG_1_FEM0_NONE     0x00            //N/A
//#define CFG_COG_1_FEM1_LOOP     0x52            //CLC1, CLC3, PWM5 high->low
#define CFG_COG_1_FEM1_LOOP     0x12            //CLC1, PWM5 high->low, CLC3 low

//Auto shutdown
#define CFG_COG_1_ASD0          0xA8            //COG is shutdown, all outputs set low
#define CFG_COG_1_ASD1          0x00            //No auto-shutdown sources

//Delays & blanking
#define CFG_COG_1_RE_DEADB      0x00            //No rising dead band
#define CFG_COG_1_FE_DEADB      0x00            //No falling dead band
#define CFG_COG_1_RE_BLANK      0x00            //No rising edge blanking
#define CFG_COG_1_FE_BLANK      0x00            //No falling edge blanking
#define CFG_COG_1_RE_PHASE      0x00            //No rising event phase delay
#define CFG_COG_1_FE_PHASE      0x00            //No falling event phase delay


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure PRG as rising ramp source for control loop
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_PRG_3_C0    0xBA            //Enabled, edge triggers, rising ramp, one-shot, no-go
#define CFG_PRG_3_C1    0x02            //Fall timing active low, rise is active high
#define CFG_PRG_3_VS    0x05            //Use DAC7 for voltage source
#define CFG_PRG_3_C2    0x00            //2uA current=0.2V/uS
#define CFG_PRG_3_RS    0x0C            //PWM5 is rising source trigger
#define CFG_PRG_3_FS    0x0C            //PWM5 is falling source trigger


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          Object definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Object to store a measured voltage with 3-digit mV resolution
typedef struct {
    unsigned int volt;                  //Voltage
    unsigned int mv;                    //Millivolts (3 decimals)
    unsigned int vmv;                   //Composite volts & mv represented as integer
    unsigned int adc;                   //Unmodified value read from ADC
} volts;

//Configuration record to control operation of inverter
//To calculate DC% from measured DC:  Config.odcp = ((Config.oldc * 100) / (CFG_PWM_5_FRQ + 1));
typedef struct {
    unsigned char sts;                  //System status code; see status codes
    unsigned char cogt;                 //Calculated COG on-time derived from function getCOGtime
    unsigned int  cogc;                 //Timer count of COG duty cycle
    unsigned int  odcp;                 //Open loop duty cycle % entered by user
    unsigned int  oldc;                 //Calculated 16-bit open loop duty cycle
    unsigned int  btim;                 //PWM11 duty cycle value (CS blanking time)
    unsigned int  amps;                 //Calculated primary current
    unsigned int  pwr;                  //Calculated primary power
    volts inv;                          //Inverter DC+ bus voltage
    volts vcc;                          //Inverter auxillary (Vcc) voltage
    volts vdd;                          //Inverter logic (Vdd) voltage)
    volts ics;                          //Inverter current sense S&H voltage
    volts tmp;                          //Inverter NTC voltage (temp)
    volts ref;                          //Reference voltage for control loop
    volts sen;                          //Vcc sense voltage for control loop
} invcf;


typedef struct {
    const char *str;
    const char *desc;
} cmd;

const cmd CmdList[] = {
    {"CE","Clear nvm Error"},
    {"DS","Display Status"},
    {"IF","Inverter oFf"},
    {"IO","Inverter On"},    
    {"MF","Monitor oFf"},
    {"MO","Monitor On"},
    {"RF","Relay oFf"},
    {"RO","Relay On"},    
    {"SA","Set autostart: <D/I/S>"},
    {"SB","Set Blanking: <#>"},
    {"SC","Set Current limit: <#>"},
    {"SD","Set Duty cycle: <%>"},
    {"SF","Sine oFf"},
    {"SL","Set Loop: <C/O>"},
    {"SO","Sine On"},
    {"SP","Set PRG dv/dt"},
    {"SR","Set vRef"},
    {"ST","Set loop Type: <A/D>"},
    {"RT","Reset"},
    {"?","Help"}
};
#define NUMOFCMDS   sizeof(CmdList) / (sizeof(const char *) * 2)
#define CMDCLRERR   0
#define CMDDSPSTS   1
#define CMDINVOFF   2
#define CMDINVON    3
#define CMDMONOFF   4
#define CMDMONON    5
#define CMDRLYOFF   6
#define CMDRLYON    7
#define CMDSETAUT   8
#define CMDSETBLK   9
#define CMDSETCUR   10
#define CMDSETDCY   11
#define CMDSINOFF   12
#define CMDSETLPM   13
#define CMDSINON    14
#define CMDSETPRG   15
#define CMDSETVRF   16
#define CMDSETTYP   17
#define CMDRSTPIC   18
#define CMDDSPHLP   19

#define CMDSTRING   0                   //Array position for command string
#define CMDDESCRP   1                   //Array position for command description
#define CMDSIZSTR   2                   //Size of command string
#define CMDLOWCAS   32                  //Add this to upper case to make lower
#define CMDWSPACE   ' '                 //White space
#define CMDCLOSLP   'c'                 //Closed loop parameter
#define CMDOPENLP   'o'                 //Open loop parameter
#define CMDENABLE   'e'                 //Enable
#define CMDDSABLE   'd'                 //Disable
#define CMDINVERT   'i'                 //Auto-start inverter only
#define CMDINSINE   's'                 //Auto-start inverter & sine
#define CMDLOOPAN   'a'                 //Analog loop
#define CMDLOOPDG   'd'                 //Digital loop


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                             Status codes
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
typedef struct {
    const unsigned char code;
    const unsigned char ledmap;
    const char *desc;
} sts;

#define STS_STOP        0x00
#define STS_RUN         0x01
#define STS_BOOT        0x02
#define STS_BUS_UVLO    0x03
#define STS_OVC_LCKO    0x04
#define STS_CHL_BSAF    0x05
#define STS_CHL_BSBF    0x06
#define STS_CPU         0x07
#define STS_COG_TIME    0x08
#define STS_SIN_CMD     0x09
#define STS_RLY_CMD     0x0A
#define STS_NVM_INV     0x0B
#define STS_HIGH_TEMP   0x0C
#define STS_OVERLOAD    0x0D
#define STS_BUS_OVLO    0x0E
#define STS_UND         0x0F            //Should always be last in list

#define STS_CODE_ERR    0x03            //Codes < value ARE NOT errors
#define STS_CODE_OK     (STS_CODE_ERR - 1)  //Codes > value ARE errors

const sts StsList[] = {
    {STS_STOP,0x01,"stop"},
    {STS_RUN,0x02,"run"},
    {STS_BOOT,0x75,"boot"},
    {STS_BUS_UVLO,0x09,"UVLO"},
    {STS_OVC_LCKO,0x11,"OC lockout"},
    {STS_CHL_BSAF,0x19,"BS A"},
    {STS_CHL_BSBF,0x21,"BS B"},
    {STS_CPU,0x29,"CPU fail"},
    {STS_COG_TIME,0x31,"low COG time"},
    {STS_SIN_CMD,0x39,"sine fail"},
    {STS_RLY_CMD,0x41,"relay fail"},
    {STS_NVM_INV,0x49,"invalid NVM"},
    {STS_HIGH_TEMP,0x51,"high temp"},
    {STS_OVERLOAD,0x59,"overload"},
    {STS_BUS_OVLO,0x61,"OVLO"},
    {STS_UND,0x79,"none"}
};


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Message, format, and label strings
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
const char *msgName = "TS350 True Sine Inverter",
        *msgCW = "Copyright (c) 2018 Brian Cornell",
        *msgErr = "ERR\n",
        *msgHuh = "\n?\n",
        *msgOk = "Ok\n",
        *msgNewl = "\n\n",
        *msgOn = "on",
        *msgOff = "off",
        //*msgOpen = "open",
        //*msgClosed = "closed",
        *msgInv = "inverter",
        *msgSine = "sine",
        *lblInv = "Inv=",
        *lblShcs = "S&H=",
        *lblTemp = "Tmp=",
        *lblVccs = "Vsen=",
        *lblDc = "DC=",
        *fmtMonvd = "%s%u.%3.3u\t",
        *fmtMonlt = "%s%u.%3.3u\n",
        *lblBtime = "BTime=",
        *lblSys = "Sys: ",
        *lblAmp = "A=",
        *lblPwr = "W=";


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                     Configure system clock & Watchdog Timer
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_OSC_C       0x78            //16MHz, FOSC config word determines clock
#define CFG_WDT_CON     0x10            //Watchdog interval=256mS, off.
#define _XTAL_FREQ      16000000        //Required to use __delayxxx macros; must match Fosc


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Function declarations.  See source file for explanation of each function.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void interrupt InterruptExec(void);
void Initialize(void);
void main(void);
void DisplayStatus(void);
void setLoop(char *);
void setDutyCycle(char *);
void setCurrentLimit(char *);
void setVref(char *);
void setPrg(char *);
bit InverterON(void);
void InverterOFF(void);
void ReadVoltage(unsigned char, unsigned char, volts *);
void CalcAvg(unsigned int, unsigned char, unsigned char, volts *);
void getCOGtime(void);
void chgDutyCycle(unsigned int, unsigned int);
unsigned char chargeBScaps(void);
bit setSine(unsigned char);
bit setRelay(unsigned char);
void eraseFlashRow(uint16_t addr);
void writeFlashRow(uint16_t, unsigned char *);
void setAutostart(char *);
void setLoopType(char *);
void setBlanking(char *);
void setLED(unsigned char, unsigned int);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Preprocessor checks.  Maintain at end of main include file.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Check for maximum current sense sample size exceeding unsigned int.
#define _ADC_OVC_VAL    ((CFG_DAC_1_VH << 8) | CFG_DAC_1_VL)
#if((_ADC_OVC_VAL * S_LM_SAMPLES) > UINT_MAX)
#error "Max current sense value _ADC_OVC_VAL * S_LM_SAMPLES exceeds unsigned int."
#endif



//End of file